package com.tencent.edu.kernel.http

import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.kernel.KernelUtil
import okhttp3.*
import java.io.IOException
import java.util.concurrent.TimeUnit


/**
 * OKHttp工具类
 * @author aidanyang
 * @date 2020-05-06
 */
object OKHttpUtil {

    private const val REFER: String = "https://ke.qq.com"
    const val TAG = "OKHttpUtil"

    private var mHttpClient: OkHttpClient = OkHttpClient.Builder()
            .connectTimeout((5 * 1000).toLong(), TimeUnit.MILLISECONDS) //链接超时
            .readTimeout((10 * 1000).toLong(), TimeUnit.MILLISECONDS) //读取超时
            .writeTimeout((10 * 1000).toLong(), TimeUnit.MILLISECONDS) //写入超时
            .retryOnConnectionFailure(true)
//                        .addInterceptor(HttpHeadInterceptor()) //应用拦截器：统一添加消息头
//                        .addNetworkInterceptor(NetworkspaceInterceptor())//网络拦截器
//                        .cache(cache)  //设置缓存
//                        .addInterceptor(LoggingInterceptor())//应用拦截器：打印日志
            .build()

    @JvmOverloads
    fun post(url: String, json: String, useCookie: Boolean = true, useRefer: Boolean = true, callback: Callback): Call {
        // 表单是json
        val mediaType = MediaType.parse("application/json;charset=utf-8")
        val body = RequestBody.create(mediaType, json)

        val builder = Request.Builder()
                .url(url)
                .post(body)

        addCookieAndRefer(useCookie, useRefer, builder)

        val request = builder.build()
        return request(request, callback)
    }

    @JvmOverloads
    fun get(url: String, useCookie: Boolean = true, useRefer: Boolean = true, callback: Callback): Call {
        val builder = Request.Builder()
                .url(url)
                .get()

        addCookieAndRefer(useCookie, useRefer, builder)

        val request = builder.build()
        return request(request, callback)
    }

    private fun addCookieAndRefer(useCookie: Boolean, useRefer: Boolean, builder: Request.Builder) {
        if (useCookie) {
            builder.addHeader("Cookie", KernelUtil.genGeneralCookie())
        }

        if (useRefer) {
            builder.addHeader("Referer", REFER)
        }
    }

    fun request(request: Request, callback: Callback): Call {
        val call = mHttpClient.newCall(request)
        call.enqueue(callback)
        return call
    }
}

class LoggingInterceptor : Interceptor {
    @Throws(IOException::class)
    override fun intercept(chain: Interceptor.Chain): Response {
        val request = chain.request()

        val t1 = System.nanoTime()
        LogUtils.ld(TAG, String.format("Sending request %s on %s%n%s",
                request.url(), chain.connection(), request.headers()))

        var response = chain.proceed(request)
        val t2 = System.nanoTime()
        LogUtils.ld(TAG, String.format("Received response for %s in %.1fms%n%s",
                response.request().url(), (t2 - t1) / 1e6, response.headers()))

        val mediaType = response.body()?.contentType()
        val content = response.body()?.string() ?: ""
        LogUtils.ld(TAG, content)

        response = response.newBuilder()
                .body(ResponseBody.create(mediaType, content))
                .build()
        return response
    }

    companion object {
        private const val TAG = "OKHttpLogger"
    }
}