package com.tencent.edu.module.knowledge

import android.text.TextUtils
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.Tips
import com.tencent.edu.kernel.http.OKHttpUtil.get
import com.tencent.edu.kernel.http.OKHttpUtil.post
import okhttp3.Call
import okhttp3.Callback
import okhttp3.Response
import org.json.JSONObject
import java.io.IOException

/**
 * 涨知识收藏请求类
 *
 * @author aidanyang
 * @date 2020-7-14
 */
class KnowledgeCollectRequester {

    private var mGetCollectStatusCall: Call? = null
    private var mSetCollectStatusCall: Call? = null

    /**
     * 获取收藏状态
     */
    fun getCollectStatus(contentId: String?, seriesId: String?, externalFlag: Int, listenerGet: GetCollectStatusListener) {
        if (mGetCollectStatusCall != null) {
            mGetCollectStatusCall?.cancel()
        }

        // contentId与seriesId互斥
        val realSeriesId = if (contentId.isNullOrBlank()) seriesId else ""

        val url = "https://ke.qq.com/cgi-node/ke/knowledge/get_collect_status" +
                "?contentId=${if (contentId.isNullOrBlank()) "" else contentId}" +
                "&externalFlag=$externalFlag" +
                "&seriesId=${if (realSeriesId.isNullOrBlank()) "" else realSeriesId}"
        LogUtils.i(TAG, "getCollectStatus request url=$url")
        mGetCollectStatusCall = get(url, callback = object : Callback {
            override fun onFailure(call: Call, e: IOException) {
                LogUtils.e(TAG, "request error=" + e.toString() + if (e.cause != null) " cause=" + e.cause.toString() else "")
                listenerGet.isCollect(false)
            }

            override fun onResponse(call: Call, response: Response) {
                if (response.body() != null) {
                    try {
                        val data = response.body()!!.string()
                        if (TextUtils.isEmpty(data)) {
                            LogUtils.w(TAG, "onResponse: body is empty")
                            listenerGet.isCollect(false)
                            return
                        }
                        LogUtils.i(TAG, "origin data:$data")
                        val jsonResp = JSONObject(data)
                        val retcode = jsonResp.optInt("retcode")
                        if (retcode != 0) {
                            LogUtils.w(TAG, "request failed, retcode=$retcode")
                            listenerGet.isCollect(false)
                            return
                        }
                        val result = jsonResp.optJSONObject("result")
                        if (result == null) {
                            LogUtils.w(TAG, "request failed, result = null")
                            return
                        }
                        val isCollect = result.optBoolean("is_collect", false)
                        listenerGet.isCollect(isCollect)
                        LogUtils.i(TAG, "request success")
                    } catch (e: Exception) {
                        e.printStackTrace()
                        LogUtils.e(TAG, "parse error=" + e.toString() + if (e.cause != null) " cause=" + e.cause.toString() else "")
                        listenerGet.isCollect(false)
                    }
                } else {
                    LogUtils.w(TAG, "onResponse: body is null")
                    listenerGet.isCollect(false)
                }
            }
        })
    }

    /**
     * 更新收藏状态
     */
    fun updateCollectStatus(contentId: String?, externalFlag: Int, type: Int, seriesId: String?, listener: UpdateCollectStatusListener) {
        if (mSetCollectStatusCall != null) {
            mSetCollectStatusCall?.cancel()
        }

        val url = "https://ke.qq.com/cgi-node/ke/knowledge/collect_content"
        val jObj = JSONObject()
        // contentId与seriesId互斥
        if (!contentId.isNullOrBlank())
            jObj.put("contentId", contentId)
        else
            jObj.put("seriesId", seriesId)

        jObj.put("externalFlag", externalFlag)
        jObj.put("type", type)

        LogUtils.i(TAG, "updateCollectStatus request url=$url")
        mSetCollectStatusCall = post(url, jObj.toString(), callback = object : Callback {
            override fun onFailure(call: Call, e: IOException) {
                LogUtils.e(TAG, "request error=" + e.toString() + if (e.cause != null) " cause=" + e.cause.toString() else "")
                listener.onError(type)
            }

            override fun onResponse(call: Call, response: Response) {
                if (response.body() != null) {
                    try {
                        val data = response.body()!!.string()
                        if (TextUtils.isEmpty(data)) {
                            LogUtils.w(TAG, "onResponse: body is empty")
                            listener.onError(type)
                            return
                        }
                        LogUtils.i(TAG, "origin data:$data")
                        val jsonResp = JSONObject(data)
                        val retcode = jsonResp.optInt("retcode")
                        if (retcode != 0) {
                            LogUtils.w(TAG, "request failed, retcode=$retcode")
                            listener.onError(type)
                            return
                        }

                        listener.onSuccess(type)
                        LogUtils.i(TAG, "request success")
                    } catch (e: Exception) {
                        e.printStackTrace()
                        LogUtils.e(TAG, "parse error=" + e.toString() + if (e.cause != null) " cause=" + e.cause.toString() else "")
                        listener.onError(type)
                    }
                } else {
                    LogUtils.w(TAG, "onResponse: body is null")
                    listener.onError(type)
                }
            }
        })
    }

    interface GetCollectStatusListener {
        fun isCollect(isCollect: Boolean)
    }

    interface UpdateCollectStatusListener {
        fun onSuccess(externalFlag: Int)
        fun onError(externalFlag: Int)
    }

    open class UpdateCollectStatusListenerImpl : UpdateCollectStatusListener {
        override fun onSuccess(type: Int) {
            Tips.showShortToast((if (type == 0) "" else "取消") + "收藏成功")
        }

        override fun onError(type: Int) {
            Tips.showShortToast((if (type == 0) "" else "取消") + "收藏失败")
        }
    }

    companion object {
        private const val TAG = "KnowledgeCollectRequester"
    }
}